import {
  CMType,
  HMType,
  HostMessage,
  Transaction,
  TransactionResponse,
  ValidationError,
} from '../types';
import {
  MessageCallback,
  MessageEventManager,
} from '../utils/MessageEventManager';

/**
 * An embedded Notabene component
 * @public
 */
export default class EmbeddedComponent {
  private _url: string;
  private _value: Partial<Transaction>;
  private _errors: ValidationError[] = [];

  private iframe?: HTMLIFrameElement;
  private eventManager: MessageEventManager;
  private ready = false;
  private modal?: HTMLDialogElement;
  // private popup?: HTMLElement;

  constructor(url: string, value: Partial<Transaction>) {
    this._url = url;
    this._value = value;
    this.eventManager = new MessageEventManager();
    this.on(CMType.INVALID, (message) => {
      if (message.type === CMType.INVALID) {
        this._errors = message.errors;
        this._value = message.value as Transaction;
      }
    });
    this.on(CMType.RESIZE, (message) => {
      if (message.type === CMType.RESIZE) {
        if (this.iframe) this.iframe.style.height = `${message.height}px`;
      }
    });
  }

  public get url(): string {
    return this._url;
  }

  public get value(): Partial<Transaction> {
    return this._value;
  }

  public get errors(): ValidationError[] {
    return this._errors;
  }

  public open() {
    document.location.href = this.url;
  }

  mount(parentId: string) {
    const parent = document.querySelector(parentId);
    if (!parent) throw new Error(`parentID ${parentId} not found`);
    this.embed(parent);
  }

  embed(parent: Element) {
    this.iframe = document.createElement('iframe');
    this.iframe.src = this.url;
    this.iframe.style.width = '100%';
    this.iframe.style.height = '100px';
    this.iframe.style.border = 'none';
    this.iframe.style.overflow = 'hidden';
    this.iframe.setAttribute('allowtransparency', 'true');
    this.iframe.style.backgroundColor = 'transparent';
    parent.appendChild(this.iframe);

    window.addEventListener('message', (event: MessageEvent) => {
      if (event.source !== this.iframe?.contentWindow) {
        return;
      }
      console.log('received message from iframe', event.data);
      this.eventManager?.setPort(event.ports[0]);
      this.ready = true;
    });

    this.iframe?.contentWindow?.focus();
  }

  send(message: HostMessage): void {
    this.eventManager.send(message);
  }

  on(messageType: string, callback: MessageCallback): void {
    this.eventManager.on(messageType, callback);
  }

  off(messageType: string, callback: MessageCallback): void {
    this.eventManager.off(messageType, callback);
  }

  update(value: Partial<Transaction>) {
    this._value = value;
    this.send({ type: HMType.UPDATE, value });
  }

  completion(): Promise<TransactionResponse> {
    return new Promise((resolve, reject) => {
      this.closeModal();
      this.on(CMType.COMPLETE, (message) => {
        resolve((message as any).response);
      });
      this.on('error', (message) => {
        reject(new Error((message as any).message));
      });
    });
  }

  // popup(): void {
  //   this.popup = document.createElement('div');
  //   this.popup.style;
  // }

  openModal(): void {
    if (this.modal) {
      this.closeModal();
    }
    this.modal = document.createElement('dialog');
    // this.modal.style.maxWidth = '600px';
    // this.modal.style.border = 'none !important';
    // this.modal.style.background = 'transparent';
    // this.modal.style.flexDirection = 'column';
    // this.modal.style.alignItems = 'end';
    // const close = document.createElement('button');
    // close.innerText = 'X';
    // close.style.fontSize = '1rem';
    // close.style.padding = '0.25rem 0.5rem';
    // close.style.display = 'block';
    // close.style.background = 'white';
    // close.style.color = '#aaa';
    // close.style.border = '#aaa';
    // close.addEventListener('click', () => this.closeModal());
    // this.modal.appendChild(close);
    // this.modal.css;
    // this.modal.style.position = 'fixed';
    // this.modal.style.bottom = '0';
    // this.modal.style.left = '0';
    // this.modal.style.right = '0';
    // this.modal.style.top = '0';
    // this.modal.style.left = '0';
    // this.modal.style.width = '100%';
    // this.modal.style.height = '100%';
    // this.modal.style.backgroundColor = 'transparent';
    // this.modal.style.display = 'flex';
    // this.modal.style.justifyContent = 'center';
    // this.modal.style.alignItems = 'center';
    document.body.appendChild(this.modal);
    this.embed(this.modal);
    this.modal.show();
    this.modal.addEventListener('click', () => {
      // console.log('click');
      this.closeModal();
    });
  }

  private closeModal(): void {
    if (this.modal) {
      console.log('closeModal');
      this.modal?.close();
      this.modal.remove();
      this.modal = undefined;
    }
  }
}
