import EmbeddedComponent from './components/EmbeddedComponent';
import type {
  Agent,
  AgentType,
  CallbackOptions,
  ComponentMessage,
  Counterparty,
  DeclarationProof,
  Deposit,
  DepositRequest,
  Destination,
  HostMessage,
  IVMS101,
  MicroTransferProof,
  OwnershipProof,
  PersonType,
  ProofStatus,
  ProofTypes,
  ScreenshotProof,
  SignatureProof,
  Status,
  Theme,
  Transaction,
  TransactionAsset,
  TransactionOptions,
  VASP,
  Wallet,
  Withdrawal,
} from './types';
import { CMType, TransactionResponse } from './types';
import { encodeObjectToFragment } from './utils/urls';

export type {
  CMType,
  ComponentMessage,
  EmbeddedComponent,
  HostMessage,
  Theme,
  Counterparty,
  Withdrawal,
  Deposit,
  Transaction,
  DepositRequest,
  TransactionResponse,
  Destination as destination,
  TransactionAsset,
  SignatureProof,
  ScreenshotProof,
  MicroTransferProof,
  DeclarationProof,
  Notabene,
  ProofStatus,
  ProofTypes,
  IVMS101,
  PersonType,
  Status,
  VASP,
  Wallet,
  Agent,
  AgentType,
  OwnershipProof,
  TransactionOptions,
};

/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
export interface NotabeneConfig {
  nodeUrl?: string;
  authToken?: string;
  uxUrl?: string;
  theme?: Theme;
  locale?: string;
}

/**
 * Primary constructor for Notabene UX elements
 *
 * @public
 */

export default class Notabene {
  private nodeUrl?: string;
  private authToken?: string;
  private uxUrl: string;
  private theme?: Theme;
  private locale?: string;

  constructor(config: NotabeneConfig) {
    this.uxUrl = config.uxUrl || 'https://connect.notabene.id';
    this.nodeUrl = config.nodeUrl;
    this.authToken = config.authToken;
    this.theme = config.theme;
    this.locale = config.locale;
  }

  public componentUrl(
    path: string,
    value: Partial<Transaction>,
    configuration?: TransactionOptions,
    callbacks?: CallbackOptions,
  ): string {
    const url = new URL(this.uxUrl);
    url.pathname = path;

    const hash = encodeObjectToFragment({
      authToken: this.authToken,
      value,
      configuration,
    });
    url.hash = hash;
    if (this.nodeUrl) url.searchParams.set('nodeUrl', this.nodeUrl);

    if (this.theme) {
      url.searchParams.set('theme', JSON.stringify(this.theme));
    }
    if (this.locale) {
      url.searchParams.set('locale', this.locale);
    }
    if (callbacks) {
      if (callbacks.callback)
        url.searchParams.set('callback_url', callbacks.callback);
      if (callbacks.redirectUri)
        url.searchParams.set('redirect_uri', callbacks.redirectUri);
    }
    return url.toString();
  }

  public createComponent(
    path: string,
    value: Partial<Transaction>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return new EmbeddedComponent(
      this.componentUrl(path, value, options, callbacks),
      value,
    );
  }

  public createWithdrawalAssist(
    value: Partial<Withdrawal>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent('withdrawal-assist', value, options, callbacks);
  }

  public createDepositAssist(
    value: Partial<Deposit>,
    options?: any,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent('deposit-assist', value, options, callbacks);
  }

  public createConnect(
    value: Partial<Withdrawal>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent('connect', value, options, callbacks);
  }

  public createDepositRequest(
    value: Partial<DepositRequest>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent('deposit-request', value, options, callbacks);
  }

  public decodeFragmentToObject(fragment: string): Record<string, string> {
    const pairs = fragment.slice(1).split('&');
    return pairs.reduce(
      (obj, pair) => {
        const [key, value] = pair.split('=');
        obj[decodeURIComponent(key)] = decodeURIComponent(value);
        return obj;
      },
      {} as Record<string, string>,
    );
  }
}
