import {
  Beneficiary,
  DateAndPlaceOfBirth,
  IVMS101,
  NationalIdentification,
  Originator,
} from './ivms/types';

export type { IVMS101 };

export type CAIP2 = `${string}:${string}`;
export type CAIP10 = `${CAIP2}:${string}`;
export type CAIP19 = `${CAIP2}/${string}:${string}`;
export type CAIP220 = string;

export type DTI = string;
export type NotabeneAsset = string;

/**
 * The asset of a transaction either a Notabene asset, a CAIP-19 asset, or a DTI.
 *
 * @public
 */

export type TransactionAsset = NotabeneAsset | CAIP19 | DTI;

export type BlockchainAddress = string;
export type TravelAddress = `ta${string}`;
export type CryptoCredential = `${string}.${string}.mastercard`;

/**
 * The destination of a transaction either a blockchain address, a CAIP-19 address, or a travel address.
 * @public
 */
export type Destination =
  | BlockchainAddress
  | CAIP10
  | CryptoCredential
  | TravelAddress;
export type Source = BlockchainAddress | CAIP10;
export type URI = string;
export type DID = `did:${string}:${string}`;

/**
 * The theme of the Notabene SDK
 * @public
 */
export type Theme = {
  primaryColor: string;
  secondaryColor: string;
  fontFamily: string;
  logo: string;
};

// export type TransactionType = 'VASP_2_VASP' | 'SELF_HOSTED';

// export type TransactionTypeAllowed =
//   | 'ALL'
//   | 'VASP_2_VASP_ONLY'
//   | 'FIRST_PARTY_ONLY';

// export type NonCustodialDeclarationType =
//   | 'SIGNATURE'
//   | 'DECLARATION'
//   | 'SIGNATURE_AND_DECLARATION';

/**
 * The type of Agent. Either a wallet or a VASP
 * @public
 */

export enum AgentType {
  PRIVATE = 'WALLET',
  VASP = 'VASP',
}

/**
 * Who is the agent acting on behalf of the counterparty
 * @public
 */
export declare interface Agent {
  did: DID;
  type: AgentType;
  logo?: string;
  url?: string;
  name?: string;
  verified?: boolean;
}
export type OwnershipProofInput = {
  type?: string | null;
  proof?: string | null;
};

export type TransactionBlockchainInfo = {
  destination?: string | null;
  origin?: string | null;
};

export type CountryType = {
  id: string;
  name: string;
};

/**
 * The type of counterparty. Either a natural person or a legal person. If the customer is the same as the counterparty then the counterparty is a self.
 * @public
 */
export enum PersonType {
  NATURAL = 'natural',
  LEGAL = 'legal',
  SELF = 'self', // first party
}

/**
 * A VASP agent acting on behalf of the counterparty
 * @public
 */
export interface VASP extends Agent {
  url?: string;
  name?: string;
}
/**
 * A wallet agent acting on behalf of the counterparty
 * @public
 */
export interface Wallet extends Agent {
  proof: OwnershipProof;
  wallet_connect_id?: string;
}

/**
 * The counterparty of a transaction.
 * @public
 */
export interface Counterparty {
  name?: string;
  accountNumber?: string;
  did?: string;
  geographicAddress?: string;
  type?: PersonType;
  verified?: boolean;
}
/**
 * A counterparty object representing a natural person
 * @public
 */
export interface NaturalPerson extends Counterparty {
  type: PersonType.NATURAL;
  nationalIdentification?: NationalIdentification;
  dateAndPlaceOfBirth?: DateAndPlaceOfBirth;
  name: string;
}

/**
 * A counterparty object representing a legal person
 * @public
 */
export interface LegalPerson extends Counterparty {
  type: PersonType.LEGAL;
  name: string;
  lei?: string;
  url?: string;
  logo?: string;
}

type OriginatorFields = {
  source?: Source;
};

type BeneficiaryFields = {
  destination?: Destination;
};

type DepositRequestFields = {
  destinations: Destination[];
};

/**
 * An abstract transaction object
 * @public
 */
export interface Transaction {
  agent: Agent;
  customer?: Counterparty;
  counterparty: Counterparty;
  asset: TransactionAsset;
  amountDecimal: number;
  // transactionBlockchainInfo?: TransactionBlockchainInfo;
  customAssetPrice?: {
    price: number;
    currency: string;
  };
}

/**
 * An object representing a withdrawal transaction
 * @public
 */
export interface Withdrawal extends BeneficiaryFields, Transaction {}

/**
 * An object representing a deposit transaction
 * @public
 */
export interface Deposit extends OriginatorFields, Transaction {}

/**
 * An object representing a request for a deposit
 * @public
 */
export interface DepositRequest extends DepositRequestFields, Transaction {}

/**
 * Configuration options for Transaction components
 * @public
 */

export interface TransactionOptions {
  microTransactionAddress?: BlockchainAddress;
  proofFallbacks?: ProofTypes[];
}

/**
 * The verification status of a transaction
 * @public
 */
export enum Status {
  EMPTY = 'empty',
  VERIFY = 'verify',
  PENDING = 'pending',
  VERIFIED = 'verified',
  BANNED = 'banned',
}

type V1Asset =
  | {
      caip19: CAIP19;
    }
  | string;

/**
 * Transaction payload suitable for calling Notabene v1 tx/create
 * @public
 */

export type V1Transaction = {
  transactionAsset: V1Asset;
  transactionAmount: string;
  originatorEqualsBeneficiary?: boolean;
  originatorVASPdid: DID;
  beneficiaryVASPdid: DID;
  beneficiaryProof: OwnershipProof;
  originator: Originator;
  beneficiary: Beneficiary;
};
/**
 * The response of a transaction
 * @public
 */

export type TransactionResponse = {
  value: Transaction;
  ivms: IVMS101;
  proof?: OwnershipProof;
  valid: boolean;
  status: Status;
  errors: ValidationError[];
  txCreate?: V1Transaction;
};

export type ValidationError = {
  attribute: string;
  message: string;
};

export type FieldProps = {
  forceDisplay?: boolean; // instead of beneficiaryDetails - defaults to false
  optional?: boolean; // bypass the jurisdiction rules validation - defaults to false
};

export type FieldName =
  | 'counterparty'
  | 'geographicAddress'
  | 'firstName'
  | 'name'
  | 'nationalIdentification'
  | 'dateAndPlaceOfBirth';

export type FieldsProps = {
  [name in FieldName]?: FieldProps;
};

/**
 * Component Message Types
 * @internal
 */
export const enum CMType {
  COMPLETE = 'complete',
  RESIZE = 'resize',
  RESULT = 'result',
  READY = 'ready',
  INVALID = 'invalid',
  MODAL = 'openModal',
  ERROR = 'error',
  CLOSE = 'closeModal',
  CANCEL = 'cancel',
}

export type Completed = {
  type: CMType.COMPLETE;
  response: TransactionResponse;
};

export type Result = {
  type: CMType.RESULT;
  reqid: RequestID;
  response: TransactionResponse;
};

export type Ready = {
  type: CMType.READY;
};

export type ResizeRequest = {
  type: CMType.RESIZE;
  height: number;
};

export type Error = {
  type: CMType.ERROR;
  message: string;
};

export type ModalRequest = {
  type: CMType.MODAL;
  url: string;
  reqid: RequestID;
};

export type CloseModal = {
  type: CMType.CLOSE;
  reqid: RequestID;
};

export type Cancel = {
  type: CMType.CANCEL;
};

export type InvalidValue = {
  type: CMType.INVALID;
  value: any;
  errors: ValidationError[];
};

/**
 * Component Message
 * @internal
 */
export type ComponentMessage =
  | Completed
  | Cancel
  | Error
  | Result
  | Ready
  | ResizeRequest
  | ModalRequest
  | InvalidValue
  | CloseModal;

export type RequestID = string;

/**
 * Host Message Types
 * @internal
 */

export const enum HMType {
  UPDATE = 'update',
  REQUEST_RESPONSE = 'requestResponse',
}

export type UpdateValue = {
  type: HMType.UPDATE;
  value: Partial<Transaction>;
};

export type RequestResponse = {
  type: HMType.REQUEST_RESPONSE;
  reqid: RequestID;
};

/**
 * Host Messages
 * @internal
 */

export type HostMessage = UpdateValue | RequestResponse;

export interface CallbackOptions {
  callback?: URI;
  redirectUri?: URI;
}

/**
 * Status of the proof
 * @public
 */
export enum ProofStatus {
  PENDING = 'pending', // Verification is pending
  FAILED = 'rejected', // Rejected
  FLAGGED = 'flagged', // Flagged for manual review
  VERIFIED = 'verified', // Verified
}

/**
 * The type of Proofs supported
 * @public
 **/
export enum ProofTypes {
  SelfDeclaration = 'self-declaration',
  PersonalSignEIP191 = 'eip-191',
  PersonalSignEIP712 = 'eip-712',
  PersonalSignBIP137 = 'bip-137',
  PersonalSignXPUB = 'xpub',
  MicroTransfer = 'microtransfer',
  Screenshot = 'screenshot',
}
/**
 * Ownership Proof
 * @public
 */
export interface OwnershipProof {
  type: ProofTypes;
  status: ProofStatus;
  did: DID;
  address: CAIP10;
}

/**
 * Ownership Proof using Message Signature
 * @public
 */
export interface SignatureProof extends OwnershipProof {
  type:
    | ProofTypes.PersonalSignEIP191
    | ProofTypes.PersonalSignEIP712
    | ProofTypes.PersonalSignBIP137
    | ProofTypes.PersonalSignXPUB;
  proof: string;
  attestation: string;
  wallet_provider: string;
}

/**
 * Ownership Proof using Self Declaration
 * @public
 */
export interface DeclarationProof extends OwnershipProof {
  type: ProofTypes.SelfDeclaration;
  attestation: string;
  confirmed: boolean;
}

/**
 * Ownership Proof using Micro Transfer
 * @public
 */
export interface MicroTransferProof extends OwnershipProof {
  type: ProofTypes.MicroTransfer;
  txhash: string;
  chain: CAIP2;
  amount: number;
}

/**
 * Ownership Proof using Screenshot
 * @public
 */
export interface ScreenshotProof extends OwnershipProof {
  type: ProofTypes.Screenshot;
  url: string;
}
